#include <WiFi.h>
#include <WebServer.h>
#include <pwmWrite.h>

const char* ssid = "AAA";
const char* password = "grelo1234";

WebServer server(80);

// Variables for button states and rotation
int arm1 = 0, arm2 = 0, grip = 0, motor_direction = 0;
Pwm pwm = Pwm();

int pump_overwrite = 0;

// Pins
const int arm1_pin = 27;
const int arm2_pin = 26;
const int grip_pin = 25;
const int pump_relay_pin = 21;
const int motor_in1_pin = 17;
const int motor_in2_pin = 16;

String generateHTML() {
    return R"rawliteral(
<!DOCTYPE html>
<html lang="en">
<head>
    <meta charset="UTF-8">
    <meta name="viewport" content="width=device-width, initial-scale=1.0">
    <title>HYDRAULIC ARM CONTROL PANEL</title>
    <link rel="stylesheet" href="https://unpkg.com/chota@latest">
    <style>
        body {
            --bg-color: #000;
            --bg-secondary-color: #131316;
            --font-color: #f5f5f5;
            background-color: var(--bg-secondary-color);
            color: var(--font-color);
            display: flex;
            flex-direction: column;
            align-items: center;
            height: 100vh;
            margin: 0;
        }
        h1 {
            text-align: center;
            margin-top: 40px;
            text-transform: uppercase;
        }
        .grid-container {
            display: grid;
            grid-template-columns: 1fr 1fr;
            gap: 10px;
            width: 80%;
            max-width: 600px;
            margin-top: 20px;
        }
        .card {
            padding: 20px;
            background-color: var(--bg-color);
            color: var(--font-color);
            text-align: left;
            display: flex;
            flex-direction: column;
            justify-content: space-between;
            user-select: none;
        }
        .card button {
            margin: 5px;
        }
        .retract {
            background-color: red;
            color: white;
        }
        .info-card {
            grid-column: 1 / span 2;
            padding: 20px;
            background-color: var(--bg-color);
            color: var(--font-color);
            text-align: left;
            user-select: none;
        }
        footer {
            margin-top: auto;
            padding: 10px;
            font-size: 0.8em;
            color: #777;
        }
        .highlighted {
            border: 2px solid #007bff;
        }
    </style>
</head>
<body>
    <h1>HYDRAULIC ARM CONTROL PANEL</h1>

    <div class="grid-container">
        <div class="card">
            <h2>Arm 1</h2>
            <button class="button primary" onmousedown="sendCommand('arm1_extend')" onmouseup="sendCommand('arm1_default')" 
                    ontouchstart="sendCommand('arm1_extend')" ontouchend="sendCommand('arm1_default')">Extend</button>
            <button class="button error" onmousedown="sendCommand('arm1_retract')" onmouseup="sendCommand('arm1_default')" 
                    ontouchstart="sendCommand('arm1_retract')" ontouchend="sendCommand('arm1_default')">Retract</button>
        </div>

        <div class="card">
            <h2>Arm 2</h2>
            <button class="button primary" onmousedown="sendCommand('arm2_extend')" onmouseup="sendCommand('arm2_default')" 
                    ontouchstart="sendCommand('arm2_extend')" ontouchend="sendCommand('arm2_default')">Extend</button>
            <button class="button error" onmousedown="sendCommand('arm2_retract')" onmouseup="sendCommand('arm2_default')" 
                    ontouchstart="sendCommand('arm2_retract')" ontouchend="sendCommand('arm2_default')">Retract</button>
        </div>

        <div class="card">
            <h2>Rotation</h2>
            <button class="button primary" onmousedown="sendCommand('rotation_right')" onmouseup="sendCommand('rotation_default')" 
                    ontouchstart="sendCommand('rotation_right')" ontouchend="sendCommand('rotation_default')">Right</button>
            <button class="button error" onmousedown="sendCommand('rotation_left')" onmouseup="sendCommand('rotation_default')" 
                    ontouchstart="sendCommand('rotation_left')" ontouchend="sendCommand('rotation_default')">Left</button>
        </div>

        <div class="card">
            <h2>Grip</h2>
            <button class="button primary" onmousedown="sendCommand('grip_extend')" onmouseup="sendCommand('grip_default')" 
                    ontouchstart="sendCommand('grip_extend')" ontouchend="sendCommand('grip_default')">Extend</button>
            <button class="button error" onmousedown="sendCommand('grip_retract')" onmouseup="sendCommand('grip_default')" 
                    ontouchstart="sendCommand('grip_retract')" ontouchend="sendCommand('grip_default')">Retract</button>
        </div>

        <div class="info-card">
            <h2>Information</h2>
            <p>Pump Pressure: <strong>0.48Mpa</strong></p>
            <p>Supply Voltage: <strong>12V</strong></p>
            <p>Pump overwrite:</p>
            <button id="pump-on" class="button primary" onclick="setPumpOverwrite(1)">On</button>
            <button id="pump-off" class="button error" onclick="setPumpOverwrite(0)">Off</button>
        </div>
    </div>

    <footer>Made by Nasser and Joon-ho</footer>

    <script>
        let pump_overwrite = 0;

        function sendCommand(command) {
            fetch(`/command?name=${command}`);
        }

        function setPumpOverwrite(value) {
            pump_overwrite = value;
            fetch(`/command?name=pump_overwrite&value=${value}`);
            updatePumpButtons();
        }

        function updatePumpButtons() {
            const onButton = document.getElementById('pump-on');
            const offButton = document.getElementById('pump-off');
            if (pump_overwrite === 1) {
                onButton.classList.add('highlighted');
                offButton.classList.remove('highlighted');
            } else {
                onButton.classList.remove('highlighted');
                offButton.classList.add('highlighted');
            }
        }

        updatePumpButtons();
    </script>
</body>
</html>
)rawliteral";
}

void handleCommand() {
    String name = server.arg("name");
    String value = server.arg("value");

    // Arm 1 controls
    if (name == "arm1_extend") arm1 = 1;
    else if (name == "arm1_retract") arm1 = 2;
    else if (name == "arm1_default") arm1 = 0;

    // Arm 2 controls
    else if (name == "arm2_extend") arm2 = 1;
    else if (name == "arm2_retract") arm2 = 2;
    else if (name == "arm2_default") arm2 = 0;

    // Grip controls
    else if (name == "grip_extend") grip = 1;
    else if (name == "grip_retract") grip = 2;
    else if (name == "grip_default") grip = 0;

    // Rotation controls
    else if (name == "rotation_right") motor_direction = 1;
    else if (name == "rotation_left") motor_direction = 2;
    else if (name == "rotation_default") motor_direction = 0;

    // Pump overwrite
    else if (name == "pump_overwrite" && value != "") pump_overwrite = value.toInt();

    // Debugging
    Serial.printf("Command received: %s, Value: %s\n", name.c_str(), value.c_str());

    server.send(200, "text/plain", "OK");
}

void setup() {
    Serial.begin(115200);
    WiFi.begin(ssid, password);
    while (WiFi.status() != WL_CONNECTED) {
        delay(500);
        Serial.print(".");
    }
    Serial.println("\nConnected to WiFi!");

    server.on("/", []() { server.send(200, "text/html", generateHTML()); });
    server.on("/command", handleCommand);
    server.begin();

    pinMode(pump_relay_pin, OUTPUT);
    digitalWrite(pump_relay_pin, LOW);

    pinMode(motor_in1_pin, OUTPUT);
    pinMode(motor_in2_pin, OUTPUT);
}

void loop() {
    server.handleClient();

    // Arm 1 control
    switch (arm1) {
        case 1: pwm.writeServo(arm1_pin, 0); break;
        case 2: pwm.writeServo(arm1_pin, 180); break;
        default: pwm.writeServo(arm1_pin, 90); break;
    }

    // Arm 2 control
    switch (arm2) {
        case 1: pwm.writeServo(arm2_pin, 0); break;
        case 2: pwm.writeServo(arm2_pin, 180); break;
        default: pwm.writeServo(arm2_pin, 90); break;
    }

    // Grip control
    switch (grip) {
        case 1: pwm.writeServo(grip_pin, 0); break;
        case 2: pwm.writeServo(grip_pin, 180); break;
        default: pwm.writeServo(grip_pin, 90); break;
    }

    // Pump control
    if (pump_overwrite == 1 || arm1 != 0 || arm2 != 0 || grip != 0) {
        digitalWrite(pump_relay_pin, HIGH);
    } else {
        digitalWrite(pump_relay_pin, LOW);
    }

    // Rotation control
    if (motor_direction == 1) { // Rotate Right
        digitalWrite(motor_in1_pin, HIGH);
        digitalWrite(motor_in2_pin, LOW);
    } else if (motor_direction == 2) { // Rotate Left
        digitalWrite(motor_in1_pin, LOW);
        digitalWrite(motor_in2_pin, HIGH);
    } else { // Stop
        digitalWrite(motor_in1_pin, LOW);
        digitalWrite(motor_in2_pin, LOW);
    }
}
